@@ -27,7 +27,7 @@
   return(ranFix)
 }
 
-.post_process_family <- function(family, ranFix, families=NULL) {
+.post_process_respfamilies <- function(family, ranFix, families=NULL) {
   if ( ! is.null(families)) {
     for (mv_it in seq_along(families)) {
       ranFix <- .post_process_family_it(family=families[[mv_it]], ranFix, char_mv_it=as.character(mv_it)) 
@@ -123,11 +123,14 @@
   beta <- coefold + dbeta
   eta <- drop(X.pv %*% beta) + offset
   eta <- .sanitize_eta(eta, y=processed$y, family=family, max=40) 
-  if (family$family=="binomial") {
-    muFREQS <- family$linkinv(eta)
-    mu <- muFREQS * processed$BinomialDen
-  } else mu <- family$linkinv(eta)
-  clik <- .calc_clik(mu=mu, phi_est=phi_est,processed=processed) 
+  ## Here I can use 
+  muetablob <- .muetafn(eta=eta, BinomialDen=processed$BinomialDen, processed=processed) 
+  ## which returns a $mu=muCOUNT in all cases.   
+  # if (family$family=="binomial") {
+  #   muFREQS <- family$linkinv(eta)
+  #   mu <- muFREQS * processed$BinomialDen
+  # } else mu <- family$linkinv(eta)
+  clik <- .calc_clik(mu=muetablob$mu, phi_est=phi_est,processed=processed) 
   if (is.infinite(clik) || is.na(clik)) {  
     gainratio <- -1
     conv_clik <- Inf
@@ -143,9 +146,64 @@
     conv_clik <- abs(dclik)/(1+abs(clik))
     gainratio <- 2*dclik/denomGainratio 
   }
-  return(list(gainratio=gainratio,clik=clik,beta=beta,eta=eta,mu=mu,conv_clik=conv_clik))
+  return(list(gainratio=gainratio,clik=clik,beta=beta,eta=eta,
+              muetablob=muetablob, # with $mu=muCOUNT
+              conv_clik=conv_clik))
 }  
 
+.do_damped_WLS_glm <- function(wX, LM_wz, damping, X.pv, clik, family, old_beta_eta, phi_est, off, processed, verbose) {
+  restarted <- FALSE
+  dampingfactor <- 2
+  while(TRUE) { 
+    if (inherits(wX,"Matrix")) {
+      LevenbergMstep_result <- .LevenbergMsolve_Matrix(wAugX=wX,LM_wAugz=LM_wz,damping=damping) 
+    } else LevenbergMstep_result <- .LevenbergMstepCallingCpp(wAugX=wX,LM_wAugz=LM_wz,damping=damping) 
+    levMblob <- .eval_gain_LevM_etaGLM(LevenbergMstep_result=LevenbergMstep_result,
+                                       X.pv=X.pv, clikold=clik, family=family,
+                                       coefold=old_beta_eta,
+                                       phi_est=phi_est, offset=off,
+                                       processed=processed)
+    gainratio <- levMblob$gainratio
+    conv_crit <- max(levMblob$conv_clik, 
+                     abs(LevenbergMstep_result$rhs)/(1+clik))
+    if (is.nan(gainratio)) {
+      break
+    } else if (gainratio>0) { ## success
+      damping <- damping * max(1/3,1-(2*gainratio-1)^3)  
+      dampingfactor <- 2
+      break
+    } else if (dampingfactor>4 ## iter not accessible for test
+               && gainratio==0) { # apparently flat deviance
+      if (conv_crit < 1e-8) { # we were at optimum
+        damping <- 1e-7
+        if (verbose["trace"]) cat("#")
+        break ## apparently flat dev
+      } else { ## measurable gradient, but we don't move => too high damping ? (if damping from previous LevM step was too high)
+        if ( ! restarted) { # condition to avoid infinite loop
+          damping <- 1e-7
+          dampingfactor <- 2
+          restarted <- TRUE
+          if (verbose["trace"]) cat("-")
+          # and continue # but this allows an  infinite loop
+        } else {
+          # hmm; well, break and diagnose...
+          if (verbose["trace"]) cat("?")
+          break
+        }
+      }
+    } else { ## failure: increase damping and continue iterations
+      damping <- dampingfactor*damping
+      dampingfactor <- dampingfactor*2
+    } 
+    if (damping>1e10) break # stop("reached damping=1e10")
+  } ## while TRUE
+  RESU <- levMblob
+  RESU$damping <- damping
+  RESU$dbetaV <- LevenbergMstep_result$dbetaV
+  return(RESU)
+}
+
+
 # called by HLfit_body for models[[1]]=="etaGLM"; tested eg by test-COMPoisson:
 .calc_etaGLMblob <- function(processed, 
                          mu, eta, muetablob, 
@@ -161,8 +219,8 @@
     X.pv <- processed$AUGI0_ZX$X.pv
     y <- processed$y
     family <- processed$family
+    LM_called <- FALSE
     damping <- 1e-7 ## as suggested by Madsen-Nielsen-Tingleff... # Smyth uses abs(mean(diag(XtWX)))/nvars
-    dampingfactor <- 2
     newclik <- .calc_clik(mu=mu,phi_est=phi_est,processed=processed) ## handles the prior.weights from processed
     for (innerj in seq_len(maxit.mean)) {
       ## breaks when Xtol_rel is reached
@@ -175,101 +233,63 @@
       } else sqrtW <- sqrt(w.resid)
       wX <- .Dvec_times_m_Matrix(sqrtW, X.pv) ## keeps colnames: important for intervalStep_glm
       szAug <- sqrtW * z1  
-      names(szAug) <- colnames(X.pv) ## also important for intervalStep_glm
+      # names(szAug) <- colnames(X.pv) ## also important for intervalStep_glm
       ## simple QR solve with LevM fallback
-      if ( ! is.null(for_intervals)) {
-        currentDy <- (for_intervals$fixeflik-newclik)
-        if (currentDy < -1e-4 && 
-            (is.null(bestlik <- processed$envir$confint_best$lik) || newclik > bestlik)) {
-          if (is.null(bestlik)) {
-            locmess <- paste("A higher",names(for_intervals$fixeflik),"was found than for the original fit.",
-                             "\nThis suggests the original fit did not fully maximize",names(for_intervals$fixeflik),
-                             "\nExpect more information at end of computation.")
-            message(locmess)
-          }
-          processed$envir$confint_best$lik <- newclik
-          processed$envir$confint_best$beta_eta <- .unscale(X.pv, old_beta_eta)
-        }
-        intervalBlob <- .intervalStep_glm(old_beta=old_beta_eta,
-                                         sXaug=wX,
-                                         szAug=szAug,
-                                         for_intervals=for_intervals,
-                                         currentlik=newclik,currentDy=currentDy)
-        beta_eta <- intervalBlob$beta
-      } else {
-        qr_X <- qr(wX,tol=spaMM.getOption("qrTolerance")) 
-        beta_eta <- .safesolve_qr_vector(qr_X, szAug)
-        beta_eta <- drop(beta_eta)
-      }
-      names(beta_eta) <- colnames(X.pv)
-      # # PROBLEM is that NaN/Inf test does not catch all divergence cases so we need this :
-      eta <- off + drop(X.pv %*% beta_eta) ## updated at each inner iteration
-      muetablob <- .muetafn(eta=eta,BinomialDen=BinomialDen,processed=processed) 
-      newclik <- .calc_clik(mu=muetablob$mu, phi_est=phi_est,processed=processed) 
-      if ( is.null(for_intervals) &&
-        (newclik < clik-1e-5 || anyNA(beta_eta) || any(is.infinite(beta_eta))) ) { 
-        ## more robust LevM
-        LM_wz <- z1*sqrtW - (wX %*% old_beta_eta)
-        restarted <- FALSE
-        while(TRUE) { 
-          if (inherits(wX,"Matrix")) {
-            LevenbergMstep_result <- .LevenbergMsolve_Matrix(wAugX=wX,LM_wAugz=LM_wz,damping=damping) 
-          } else LevenbergMstep_result <- .LevenbergMstepCallingCpp(wAugX=wX,LM_wAugz=LM_wz,damping=damping) 
-          levMblob <- .eval_gain_LevM_etaGLM(LevenbergMstep_result=LevenbergMstep_result,
-                                         X.pv=X.pv, clikold=clik, family=family,
-                                         coefold=old_beta_eta,
-                                         phi_est=phi_est, offset=off,
-                                         processed=processed)
-          gainratio <- levMblob$gainratio
-          conv_crit <- max(levMblob$conv_clik, 
-                           abs(LevenbergMstep_result$rhs)/(1+clik))
-          if (is.nan(levMblob$gainratio)) {
-            break
-          } else if (gainratio>0) { ## success
-            damping <- damping * max(1/3,1-(2*levMblob$gainratio-1)^3)  
-            dampingfactor <- 2
-            dbetaV <- LevenbergMstep_result$dbetaV
-            beta_eta <- levMblob$beta 
-            eta <- off + drop(X.pv %*% beta_eta) ## updated at each inner iteration
-            muetablob <- .muetafn(eta=eta,BinomialDen=BinomialDen,processed=processed) 
-            newclik <- levMblob$clik
-            break
-          } else if (dampingfactor>4 ## iter not accessible for test
-                     && gainratio==0) { # apparently flat deviance
-            dbetaV <- LevenbergMstep_result$dbetaV ## ...hmm first test in test-formula_env requires this...
-            if (conv_crit < 1e-8) { # we were at optimum
-              damping <- 1e-7
-              if (verbose["trace"]) cat("#")
-              break ## apparently flat dev
-            } else { ## measurable gradient, but we don't move => too high damping ? (if damping from previous LevM step was too high)
-              if ( ! restarted) { # condition to avoid infinite loop
-                damping <- 1e-7
-                dampingfactor <- 2
-                restarted <- TRUE
-                if (verbose["trace"]) cat("-")
-                # and continue # but this allows an  infinite loop
-              } else {
-                # hmm; well, break and diagnose...
-                if (verbose["trace"]) cat("?")
-                break
-              }
+      if ( ! is.null(for_intervals) || ! LM_called) {
+        if ( ! is.null(for_intervals)) {
+          currentDy <- (for_intervals$fixeflik-newclik)
+          if (currentDy < -1e-4 && 
+              (is.null(bestlik <- processed$envir$confint_best$lik) || newclik > bestlik)) {
+            if (is.null(bestlik)) {
+              locmess <- paste("A higher",names(for_intervals$fixeflik),"was found than for the original fit.",
+                               "\nThis suggests the original fit did not fully maximize",names(for_intervals$fixeflik),
+                               "\nExpect more information at end of computation.")
+              message(locmess)
             }
-          } else { ## failure: increase damping and continue iterations
-            damping <- dampingfactor*damping
-            dampingfactor <- dampingfactor*2
-          } 
-          if (damping>1e10) break # stop("reached damping=1e10")
-        } ## while TRUE
+            processed$envir$confint_best$lik <- newclik
+            processed$envir$confint_best$beta_eta <- .unscale(X.pv, old_beta_eta)
+          }
+          intervalBlob <- .intervalStep_glm(old_beta=old_beta_eta,
+                                            sXaug=wX,
+                                            szAug=szAug,
+                                            for_intervals=for_intervals,
+                                            currentlik=newclik,currentDy=currentDy)
+          beta_eta <- intervalBlob$beta
+        } else if ( ! LM_called)  {
+          qr_X <- qr(wX,tol=spaMM.getOption("qrTolerance")) 
+          beta_eta <- .safesolve_qr_vector(qr_X, szAug)
+          beta_eta <- drop(beta_eta)
+        }
+        names(beta_eta) <- colnames(X.pv)
+        # # PROBLEM is that NaN/Inf test does not catch all divergence cases so we need this :
+        eta <- off + drop(X.pv %*% beta_eta) ## updated at each inner iteration
+        muetablob <- .muetafn(eta=eta,BinomialDen=BinomialDen,processed=processed) 
+        newclik <- .calc_clik(mu=muetablob$mu, phi_est=phi_est,processed=processed) 
+      }  
+      if ( is.null(for_intervals) &&
+        (LM_called || # always use LevM when it has been called before: motivated by spaMM_glm example, cf email Alex, 01/06/2022, 14:13
+         newclik < clik-1e-5 || anyNA(beta_eta) || any(is.infinite(beta_eta))) ) { 
+        ## more robust LevM
+        LM_called <- TRUE
+        LM_wz <- z1*sqrtW - (wX %*% old_beta_eta)
+        damped_WLS_blob <- .do_damped_WLS_glm(wX, LM_wz, damping, X.pv, clik, family, old_beta_eta, phi_est, off, processed, verbose)
+        beta_eta <- damped_WLS_blob$beta 
+        eta <- damped_WLS_blob$eta #off + drop(X.pv %*% beta_eta) ## updated at each inner iteration
+        muetablob <- damped_WLS_blob$muetablob # .muetafn(eta=eta,BinomialDen=BinomialDen,processed=processed) 
+        newclik <- damped_WLS_blob$clik
+        damping <- damped_WLS_blob$damping
+        dbetaV <- damped_WLS_blob$dbetaV
       } else dbetaV <- beta_eta - old_beta_eta
       mu <- muetablob$mu ## needed to update z1
       w.resid <- .calc_w_resid(muetablob$GLMweights,phi_est) ## 'weinu', must be O(n) in all cases
       if (verbose["trace"]) {
         print(paste0("Inner iteration ",innerj))
         print_err <- c(beta_eta=beta_eta)
-        if (innerj>1) print_err <- c(norm.dbetaV=sqrt(sum(dbetaV^2)),print_err)
+        if (innerj>1L) print_err <- c(norm.dbetaV=sqrt(sum(dbetaV^2)),print_err)
         print(print_err)
         print("================================================")
       } 
+      #if(innerj>600) browser()
       if (maxit.mean>1 && (damping>1e100 || mean(abs(dbetaV)) < Xtol_rel)) break
       #### done with one inner iteration
       old_beta_eta <- beta_eta
@@ -279,3 +299,70 @@
                 sXaug=structure(NA,class="LM or GLM")))
   }
 
+.calc_std_leverages <- function(models, need_ranefPars_estim, phi.Fix, auglinmodblob, n_u_h, nobs, processed, w.resid, u_h, 
+                                need_simple_lambda, muetablob, wranefblob, ZAL, lambda_est, cum_n_u_h, 
+                                lcrandfamfam=attr(processed$rand.families,"lcrandfamfam"), phi_est) {
+  hatvals <- NULL 
+  if (models[[1]]=="etaHGLM") {
+    if (need_ranefPars_estim || .anyNULL(phi.Fix)) {
+      hatval <- .get_hatvalues_MM(auglinmodblob$sXaug,X.Re=processed$X.Re, auglinmodblob$weight_X)
+      hatvals <- list(
+        ranef=hatval[seq(n_u_h)],  ## for the ranef residuals (lambda)
+        resid=hatval[(n_u_h+1L):(n_u_h+nobs)] ## for the error residuals (phi)
+      )
+    }
+  } else if (.anyNULL(phi.Fix)) { ## phi estim in GLM fitted by ML. 
+    hatvals <- list(resid=.get_hatvalues_FM(processed$X.Re, augX=processed$AUGI0_ZX$X.pv, w.resid))
+  }
+  ## (HL[2]=0, HL[3]=0): previous hat matrix -> p 
+  ## (HL[2]=0, HL[3]=1): notEQL -> tilde(p), (HL[2]=1 && ): full correction -> q 
+  ## (HL[2]=1, HL[3]=1): full correction -> q 
+  leverages <- .hatvals2std_lev(
+    hatvals, #### base from hat matrix
+    sXaug=auglinmodblob$sXaug, anynull_phi.Fix=.anyNULL(phi.Fix), u_h=u_h, 
+    need_simple_lambda=need_simple_lambda, muetablob=muetablob, wranefblob=wranefblob, nobs=nobs, ZAL=ZAL, 
+    w.resid=w.resid, 
+    psi_M=processed$psi_M, lambda_est=lambda_est, cum_n_u_h=cum_n_u_h, lcrandfamfam=lcrandfamfam, 
+    nrand=length(lcrandfamfam), phi_est=phi_est,
+    processed=processed)
+  leverages
+}
+
+.calc_APHLs_GLM <- function(processed, w.resid, clik) {
+  ## ML: X.Re non NULL mais ncol(X.Re)=0
+  X.REML <- processed$X.Re
+  if (is.null(X.REML)) {X.REML <- processed$AUGI0_ZX$X.pv} ## REML standard
+  if ( ncol(X.REML) ) { ## REML standard || REML non standard
+    Md2hdb2 <- .ZtWZwrapper(X.REML,w.resid)
+    ladb <- .LogAbsDetWrap(Md2hdb2,logfac=-log(2*pi))
+    p_bv <- clik - ladb/2
+    if ( ! is.null(X_scale <- attr(processed$AUGI0_ZX$X.pv,"scaled:scale"))) p_bv <- p_bv -sum(log(X_scale))
+    APHLs <- list(p_v=clik, p_bv=p_bv)
+  } else APHLs <- list(p_v=clik, p_bv=clik)
+  APHLs
+}
+
+.get_fixed_adjacency_info <- function(whichadj, LMatrices, cum_n_u_h, corr_est, ranFix, init.HLfit) {
+  test_inner_estim_rho <- (length(whichadj) && ! is.null(adjd <- attr(LMatrices[[whichadj]],"symsvd")$adjd))
+  if (test_inner_estim_rho) {  
+    u.range <- (cum_n_u_h[whichadj]+1L):cum_n_u_h[whichadj+1L]
+    adj_rho <- corr_est$rho
+    if (is.null(adj_rho)) adj_rho <- .getPar(ranFix,"rho") ## could this occur with test_inner_estim_rho ?
+    if (is.null(adj_rho)) adj_rho <- init.HLfit$rho
+    if ( ! is.null(adj_rho)) fixed_adjacency_info <- list(whichadj=whichadj, u.range=u.range, coeffs=1/(1-adj_rho*adjd))
+  } else fixed_adjacency_info <- NULL
+  fixed_adjacency_info
+}
+
+.get_init_beta <- function(processed, pforpv, init.HLfit, X.pv=processed$AUGI0_ZX$X.pv) {
+  beta_eta <- processed$port_env$port_fit_values$fixef # scaled
+  if (is.null(beta_eta)) { # then we look at user value and scale it <=> user values must be unscaled.
+    if (pforpv) {
+      beta_eta <- init.HLfit$fixef
+      if ( ! is.null(beta_eta) && ! is.null(attr(X.pv,"scaled:scale"))) {
+        beta_eta <- .scale(beta=beta_eta,X=X.pv)
+      }
+    } else beta_eta <- numeric(0L) # don't leave it NULL so that we don't try to get it from inits_by_glm.
+  } 
+  beta_eta
+}
