@@ -51,6 +51,14 @@ if (rngcheck) suppressWarnings(RNGkind("Mersenne-Twister", "Inversion", "Roundin
 set.seed(123)
 d1 <- rSample(nb=40,rho=3,sigma2_u=0.5,resid=0.5,intercept=-1,slope=0.1)
 if (rngcheck) RNGkind("Mersenne-Twister", "Inversion", "Rejection"  )
+# Rnews: The included LAPACK sources have been updated to 3.10.1. ... 
+# ... Using 3.10.x may give some different signs from earlier versions in SVDs or eigendecompositions... 
+# => MASS::mvrnorm affected.
+d1$obs <- c(1.56404438213562,0.838761745570047,1.18539152814119,2.77609094294087,-0.362117933355623,1.05566922884628,1.04891131933423,-1.36129005102859,
+            3.14126410723178,-0.267510545755898,1.68242492955138,-0.294886401760225,1.70572380683716,2.25124059611453,-0.850815921177352,0.439615106586318,
+            2.0392252364533,2.42453461515297,-0.53309120933375,-0.554803498441966,1.71344222448971,1.43407925526473,1.27070104772916,-0.557869677646131,
+            1.29371981088591,1.98292535259867,-0.440298987269969,0.493786122749392,0.0552051443241781,0.943700535986965,-0.0148351370346089,0.260693893323206,
+            2.45333962421848,0.276596082599056,-1.53290736864249,1.88815293467526,-1.90295848044005,2.84485580265504,-0.514747120202024,-2.44239024049091)
 
 HLMf <- fitme(obs~pred+Matern(1|x+y),init=list(rho=59.11287),fixed=list(nu=48.96201,phi=0.447761,lambda=0.3697),data=d1,method="ML")
 ci <- confint(HLMf,"pred",verbose=FALSE) 
